@@ -1,20 +1,13 @@
-from math import atan2
-import numpy as np
-
 from functools import partial
+from math import atan2
+from typing import Optional
 
 import dask.array as da
-
+import numpy as np
+import xarray as xr
 from numba import cuda
 
-import xarray as xr
-
-from xrspatial.utils import ngjit
-from xrspatial.utils import cuda_args
-from xrspatial.utils import ArrayTypeFunctionMapping
-from xrspatial.utils import not_implemented_func
-
-from typing import Optional
+from xrspatial.utils import ArrayTypeFunctionMapping, cuda_args, ngjit, not_implemented_func
 
 # 3rd-party
 try:
@@ -172,6 +165,7 @@ def aspect(agg: xr.DataArray,
     --------
     Aspect works with NumPy backed xarray DataArray
     .. sourcecode:: python
+
         >>> import numpy as np
         >>> import xarray as xr
         >>> from xrspatial import aspect
@@ -207,6 +201,7 @@ def aspect(agg: xr.DataArray,
 
     Aspect works with Dask with NumPy backed xarray DataArray
     .. sourcecode:: python
+
         >>> import dask.array as da
         >>> data_da = da.from_array(data, chunks=(3, 3))
         >>> raster_da = xr.DataArray(data_da, dims=['y', 'x'], name='raster_da')
@@ -232,6 +227,7 @@ def aspect(agg: xr.DataArray,
     Aspect works with CuPy backed xarray DataArray.
     Make sure you have a GPU and CuPy installed to run this example.
     .. sourcecode:: python
+
         >>> import cupy
         >>> data_cupy = cupy.asarray(data)
         >>> raster_cupy = xr.DataArray(data_cupy, dims=['y', 'x'])
