@@ -1,21 +1,16 @@
-from math import sqrt
-import numpy as np
-import numba as nb
-
-from numba import cuda
-
-
-import xarray as xr
-from xarray import DataArray
-import dask.array as da
-
-from xrspatial.utils import cuda_args
-from xrspatial.utils import ngjit
-from xrspatial.utils import ArrayTypeFunctionMapping
-from xrspatial.utils import validate_arrays
-from xrspatial.utils import not_implemented_func
-
 import warnings
+from math import sqrt
+
+import dask.array as da
+import numba as nb
+import numpy as np
+import xarray as xr
+from numba import cuda
+from xarray import DataArray
+
+from xrspatial.utils import (ArrayTypeFunctionMapping, cuda_args, ngjit, not_implemented_func,
+                             validate_arrays)
+
 # 3rd-party
 try:
     import cupy
@@ -106,8 +101,10 @@ def arvi(nir_agg: xr.DataArray,
     Examples
     --------
     In this example, we'll use data available in xrspatial.datasets
+
     .. plot::
        :include-source:
+
         >>> from xrspatial.datasets import get_data
         >>> data = get_data('sentinel-2')  # Open Example Data
         >>> nir = data['NIR']
@@ -122,6 +119,7 @@ def arvi(nir_agg: xr.DataArray,
         >>> arvi_agg.plot(aspect=2, size=4)
 
     .. sourcecode:: python
+
         >>> y1, x1, y2, x2 = 100, 100, 103, 104
         >>> print(nir[y1:y2, x1:x2].data)
         [[1519. 1504. 1530. 1589.]
@@ -270,6 +268,7 @@ def evi(nir_agg: xr.DataArray,
         >>> evi_agg.plot(aspect=2, size=4)
 
     .. sourcecode:: python
+
         >>> y, x = 100, 100
         >>> m, n = 3, 4
         >>> print(nir[y1:y2, x1:x2].data)
@@ -409,6 +408,7 @@ def gci(nir_agg: xr.DataArray,
         >>> gci_agg.plot(aspect=2, size=4)
 
     .. sourcecode:: python
+
         >>> y1, x1, y2, x2 = 100, 100, 103, 104
         >>> print(nir[y1:y2, x1:x2].data)
         [[1519. 1504. 1530. 1589.]
@@ -472,6 +472,7 @@ def nbr(nir_agg: xr.DataArray,
     --------
     .. plot::
        :include-source:
+
         >>> from xrspatial.datasets import get_data
         >>> data = get_data('sentinel-2')  # Open Example Data
         >>> nir = data['NIR']
@@ -484,6 +485,7 @@ def nbr(nir_agg: xr.DataArray,
         >>> nbr_agg.plot(aspect=2, size=4)
 
     .. sourcecode:: python
+
         >>> y1, x1, y2, x2 = 100, 100, 103, 104
         >>> print(nir[y1:y2, x1:x2].data)
         [[1519. 1504. 1530. 1589.]
@@ -554,6 +556,7 @@ def nbr2(swir1_agg: xr.DataArray,
     --------
     .. plot::
        :include-source:
+
         >>> from xrspatial.datasets import get_data
         >>> data = get_data('sentinel-2')  # Open Example Data
         >>> swir1 = data['SWIR1']
@@ -566,6 +569,7 @@ def nbr2(swir1_agg: xr.DataArray,
         >>> nbr2_agg.plot(aspect=2, size=4)
 
     .. sourcecode:: python
+
         >>> y1, x1, y2, x2 = 100, 100, 103, 104
         >>> print(swir1[y1:y2, x1:x2].data)
         [[2092. 2242. 2333. 2382.]
@@ -630,7 +634,7 @@ def ndvi(nir_agg: xr.DataArray,
     --------
     .. plot::
        :include-source:
-        >>> from xrspatial.datasets import get_data        
+        >>> from xrspatial.datasets import get_data
         >>> data = get_data('sentinel-2')  # Open Example Data
         >>> nir = data['NIR']
         >>> red = data['Red']
@@ -642,6 +646,7 @@ def ndvi(nir_agg: xr.DataArray,
         >>> ndvi_agg.plot(aspect=2, size=4)
 
     .. sourcecode:: python
+
         >>> y1, x1, y2, x2 = 100, 100, 103, 104
         >>> print(nir[y1:y2, x1:x2].data)
         [[1519. 1504. 1530. 1589.]
@@ -710,6 +715,7 @@ def ndmi(nir_agg: xr.DataArray,
     --------
     .. plot::
        :include-source:
+
         >>> from xrspatial.datasets import get_data
         >>> data = get_data('sentinel-2')  # Open Example Data
         >>> nir = data['NIR']
@@ -722,6 +728,7 @@ def ndmi(nir_agg: xr.DataArray,
         >>> ndmi_agg.plot(aspect=2, size=4)
 
     .. sourcecode:: python
+
         >>> y1, x1, y2, x2 = 100, 100, 103, 104
         >>> print(nir[y1:y2, x1:x2].data)
         [[1519. 1504. 1530. 1589.]
@@ -892,6 +899,7 @@ def savi(nir_agg: xr.DataArray,
     --------
     .. plot::
        :include-source:
+
         >>> from xrspatial.datasets import get_data
         >>> data = get_data('sentinel-2')  # Open Example Data
         >>> nir = data['NIR']
@@ -904,6 +912,7 @@ def savi(nir_agg: xr.DataArray,
         >>> savi_agg.plot(aspect=2, size=4)
 
     .. sourcecode:: python
+
         >>> print(nir[y1:y2, x1:x2].data)
         [[1519. 1504. 1530. 1589.]
          [1491. 1473. 1542. 1609.]
@@ -1020,6 +1029,7 @@ def sipi(nir_agg: xr.DataArray,
     --------
     .. plot::
        :include-source:
+
         >>> from xrspatial.datasets import get_data
         >>> data = get_data('sentinel-2')  # Open Example Data
         >>> nir = data['NIR']
@@ -1034,6 +1044,7 @@ def sipi(nir_agg: xr.DataArray,
         >>> sipi_agg.plot(aspect=2, size=4)
 
     .. sourcecode:: python
+
         >>> y1, x1, y2, x2 = 100, 100, 103, 104
         >>> print(nir[y1:y2, x1:x2].data)
         [[1519. 1504. 1530. 1589.]
@@ -1357,6 +1368,7 @@ def true_color(r, g, b, nodata=1, c=10.0, th=0.125, name='true_color'):
     --------
     .. plot::
        :include-source:
+
         >>> from xrspatial.datasets import get_data
         >>> data = get_data('sentinel-2')  # Open Example Data
         >>> red = data['Red']
