@@ -11,18 +11,13 @@ except ImportError:
         ndarray = False
 
 import dask.array as da
-
-from numba import cuda
-
 import numpy as np
 import xarray as xr
+from numba import cuda
 
 # local modules
-from xrspatial.utils import cuda_args
-from xrspatial.utils import get_dataarray_resolution
-from xrspatial.utils import ngjit
-from xrspatial.utils import ArrayTypeFunctionMapping
-from xrspatial.utils import not_implemented_func
+from xrspatial.utils import (ArrayTypeFunctionMapping, cuda_args, get_dataarray_resolution, ngjit,
+                             not_implemented_func)
 
 
 @ngjit
@@ -142,6 +137,7 @@ def slope(agg: xr.DataArray,
     Examples
     --------
     .. sourcecode:: python
+
         >>> import numpy as np
         >>> import xarray as xr
         >>> from xrspatial import slope
