@@ -4,8 +4,7 @@ import pytest
 from xrspatial import aspect
 from xrspatial.tests.general_checks import (assert_nan_edges_effect, assert_numpy_equals_cupy,
                                             assert_numpy_equals_dask_numpy, create_test_raster,
-                                            general_output_checks)
-from xrspatial.utils import doesnt_have_cuda
+                                            cuda_and_cupy_available, general_output_checks)
 
 
 def input_data(backend='numpy'):
@@ -70,7 +69,7 @@ def test_numpy_equals_dask_random_data(random_data):
     assert_numpy_equals_dask_numpy(numpy_agg, dask_agg, aspect)
 
 
-@pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
+@cuda_and_cupy_available
 def test_numpy_equals_cupy_qgis_data():
     # compare using the data run through QGIS
     numpy_agg = input_data()
@@ -78,7 +77,7 @@ def test_numpy_equals_cupy_qgis_data():
     assert_numpy_equals_cupy(numpy_agg, cupy_agg, aspect)
 
 
-@pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
+@cuda_and_cupy_available
 @pytest.mark.parametrize("size", [(2, 4), (10, 15)])
 @pytest.mark.parametrize(
     "dtype", [np.int32, np.int64, np.uint32, np.uint64, np.float32, np.float64])
