@@ -7,8 +7,9 @@ from xrspatial import mean
 from xrspatial.convolution import (annulus_kernel, calc_cellsize, circle_kernel, convolution_2d,
                                    convolve_2d, custom_kernel)
 from xrspatial.focal import apply, focal_stats, hotspots
-from xrspatial.tests.general_checks import create_test_raster, general_output_checks
-from xrspatial.utils import doesnt_have_cuda, ngjit
+from xrspatial.tests.general_checks import (create_test_raster, cuda_and_cupy_available,
+                                            general_output_checks)
+from xrspatial.utils import ngjit
 
 
 def _do_sparse_array(data_array):
@@ -57,7 +58,7 @@ def test_mean_transfer_function_cpu():
     )
 
 
-@pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
+@cuda_and_cupy_available
 def test_mean_transfer_function_gpu_equals_cpu():
 
     import cupy
@@ -134,6 +135,20 @@ def convolution_kernel_annulus_2_2_1():
     return expected_result
 
 
+@pytest.fixture
+def convolution_custom_kernel():
+    kernel = np.array([[1, 0, 0], [1, 1, 0], [1, 0, 0]])
+    expected_result = np.array([
+        [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan],
+        [np.nan, 2., 3., 3., 4., np.nan],
+        [np.nan, 4., np.nan, np.nan, np.nan, np.nan],
+        [np.nan, 4., np.nan, np.nan, np.nan, np.nan],
+        [np.nan, 4., np.nan, np.nan, np.nan, np.nan],
+        [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan]
+    ])
+    return kernel, expected_result
+
+
 def test_kernel_custom_kernel_invalid_type():
     kernel = [1, 0, 0]  # only arrays are accepted, not lists
     with pytest.raises(ValueError):
@@ -158,16 +173,18 @@ def test_kernel(kernel_circle_1_1_1, kernel_annulus_2_2_2_1):
 
 def test_convolution_numpy(
     convolve_2d_data,
+    convolution_custom_kernel,
     kernel_circle_1_1_1,
     convolution_kernel_circle_1_1_1,
     kernel_annulus_2_2_2_1,
     convolution_kernel_annulus_2_2_1
 ):
-    kernel_custom = np.ones((1, 1))
+    kernel_custom, expected_result_custom = convolution_custom_kernel
     result_kernel_custom = convolve_2d(convolve_2d_data, kernel_custom)
     assert isinstance(result_kernel_custom, np.ndarray)
-    # kernel is [[1]], thus the result equals input data
-    np.testing.assert_allclose(result_kernel_custom, convolve_2d_data, equal_nan=True)
+    np.testing.assert_allclose(
+        result_kernel_custom, expected_result_custom, equal_nan=True
+    )
 
     result_kernel_circle = convolve_2d(convolve_2d_data, kernel_circle_1_1_1)
     assert isinstance(result_kernel_circle, np.ndarray)
@@ -184,17 +201,20 @@ def test_convolution_numpy(
 
 def test_convolution_dask_numpy(
     convolve_2d_data,
+    convolution_custom_kernel,
     kernel_circle_1_1_1,
     convolution_kernel_circle_1_1_1,
     kernel_annulus_2_2_2_1,
     convolution_kernel_annulus_2_2_1
 ):
     dask_agg = create_test_raster(convolve_2d_data, backend='dask+numpy')
-    kernel_custom = np.ones((1, 1))
+
+    kernel_custom, expected_result_custom = convolution_custom_kernel
     result_kernel_custom = convolution_2d(dask_agg, kernel_custom)
     assert isinstance(result_kernel_custom.data, da.Array)
-    # kernel is [[1]], thus the result equals input data
-    np.testing.assert_allclose(result_kernel_custom.compute(), convolve_2d_data, equal_nan=True)
+    np.testing.assert_allclose(
+        result_kernel_custom.compute(), expected_result_custom, equal_nan=True
+    )
 
     result_kernel_circle = convolution_2d(dask_agg, kernel_circle_1_1_1)
     assert isinstance(result_kernel_circle.data, da.Array)
@@ -209,9 +229,10 @@ def test_convolution_dask_numpy(
     )
 
 
-@pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
+@cuda_and_cupy_available
 def test_2d_convolution_gpu(
     convolve_2d_data,
+    convolution_custom_kernel,
     kernel_circle_1_1_1,
     convolution_kernel_circle_1_1_1,
     kernel_annulus_2_2_2_1,
@@ -220,11 +241,12 @@ def test_2d_convolution_gpu(
     import cupy
     cupy_data = cupy.asarray(convolve_2d_data)
 
-    kernel_custom = np.ones((1, 1))
+    kernel_custom, expected_result_custom = convolution_custom_kernel
     result_kernel_custom = convolve_2d(cupy_data, kernel_custom)
     assert isinstance(result_kernel_custom, cupy.ndarray)
-    # kernel is [[1]], thus the result equals input data
-    np.testing.assert_allclose(result_kernel_custom.get(), convolve_2d_data, equal_nan=True)
+    np.testing.assert_allclose(
+        result_kernel_custom.get(), expected_result_custom, equal_nan=True
+    )
 
     result_kernel_circle = convolve_2d(cupy_data, kernel_circle_1_1_1)
     assert isinstance(result_kernel_circle, cupy.ndarray)
@@ -297,62 +319,46 @@ def test_apply_dask_numpy(data_apply):
     general_output_checks(dask_numpy_agg, dask_numpy_apply, expected_result)
 
 
-@pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
-def test_apply_gpu(data_apply):
-    data, kernel, expected_result = data_apply
-    # cupy case
-    cupy_agg = create_test_raster(data, backend='cupy')
-    cupy_apply = apply(cupy_agg, kernel, func_zero)
-    general_output_checks(cupy_agg, cupy_apply, expected_result)
-
-    # dask + cupy case not implemented
-    dask_cupy_agg = create_test_raster(data, backend='dask+cupy')
-    with pytest.raises(NotImplementedError) as e_info:
-        apply(dask_cupy_agg, kernel, func_zero)
-        assert e_info
-
-
 @pytest.fixture
 def data_focal_stats():
     data = np.arange(16).reshape(4, 4)
-    cellsize = (1, 1)
-    kernel = circle_kernel(*cellsize, 1.5)
+    kernel = custom_kernel(np.array([[1, 0, 0], [0, 1, 0], [0, 0, 0]]))
     expected_result = np.asarray([
         # mean
-        [[1.66666667, 2., 3., 4.],
-         [4.25, 5., 6., 6.75],
-         [8.25, 9., 10., 10.75],
-         [11., 12., 13., 13.33333333]],
+        [[0, 1, 2, 3.],
+         [4, 2.5,  3.5,  4.5],
+         [8, 6.5,  7.5,  8.5],
+         [12, 10.5,  11.5,  12.5]],
         # max
-        [[4., 5., 6., 7.],
-         [8., 9., 10., 11.],
-         [12., 13., 14., 15.],
-         [13., 14., 15., 15.]],
+        [[0, 1, 2, 3.],
+         [4, 5, 6, 7.],
+         [8, 9, 10, 11.],
+         [12, 13, 14, 15.]],
         # min
-        [[0., 0., 1., 2.],
-         [0., 1., 2., 3.],
-         [4., 5., 6., 7.],
-         [8., 9., 10., 11.]],
+        [[0, 1, 2, 3.],
+         [4, 0, 1, 2.],
+         [8, 4, 5, 6.],
+         [12, 8, 9, 10.]],
         # range
-        [[4., 5., 5., 5.],
-         [8., 8., 8., 8.],
-         [8., 8., 8., 8.],
-         [5., 5., 5., 4.]],
+        [[0, 0, 0, 0.],
+         [0, 5, 5, 5.],
+         [0, 5, 5, 5.],
+         [0, 5, 5, 5.]],
         # std
-        [[1.69967317, 1.87082869, 1.87082869, 2.1602469],
-         [2.86138079, 2.60768096, 2.60768096, 2.86138079],
-         [2.86138079, 2.60768096, 2.60768096, 2.86138079],
-         [2.1602469, 1.87082869, 1.87082869, 1.69967317]],
+        [[0, 0, 0, 0.],
+         [0, 2.5,  2.5,  2.5],
+         [0, 2.5,  2.5,  2.5],
+         [0, 2.5,  2.5,  2.5]],
         # var
-        [[2.88888889, 3.5, 3.5, 4.66666667],
-         [8.1875, 6.8, 6.8, 8.1875],
-         [8.1875, 6.8, 6.8, 8.1875],
-         [4.66666667, 3.5, 3.5, 2.88888889]],
+        [[0, 0, 0, 0.],
+         [0, 6.25, 6.25, 6.25],
+         [0, 6.25, 6.25, 6.25],
+         [0, 6.25, 6.25, 6.25]],
         # sum
-        [[5., 8., 12., 12.],
-         [17., 25., 30., 27.],
-         [33., 45., 50., 43.],
-         [33., 48., 52., 40.]]
+        [[0, 1, 2, 3.],
+         [4, 5, 7, 9.],
+         [8, 13, 15, 17.],
+         [12, 21, 23, 25.]]
     ])
     return data, kernel, expected_result
 
@@ -376,6 +382,16 @@ def test_focal_stats_dask_numpy(data_focal_stats):
     )
 
 
+@cuda_and_cupy_available
+def test_focal_stats_gpu(data_focal_stats):
+    data, kernel, expected_result = data_focal_stats
+    cupy_agg = create_test_raster(data, backend='cupy')
+    cupy_focalstats = focal_stats(cupy_agg, kernel)
+    general_output_checks(
+        cupy_agg, cupy_focalstats, verify_attrs=False, expected_results=expected_result
+    )
+
+
 @pytest.fixture
 def data_hotspots():
     data = np.asarray([
@@ -420,22 +436,46 @@ def test_hotspots_numpy(data_hotspots):
     data, kernel, expected_result = data_hotspots
     numpy_agg = create_test_raster(data)
     numpy_hotspots = hotspots(numpy_agg, kernel)
-    general_output_checks(numpy_agg, numpy_hotspots, expected_result)
+    general_output_checks(numpy_agg, numpy_hotspots, expected_result, verify_attrs=False)
+    # validate attrs
+    assert numpy_hotspots.shape == numpy_agg.shape
+    assert numpy_hotspots.dims == numpy_agg.dims
+    for coord in numpy_agg.coords:
+        np.testing.assert_allclose(
+            numpy_hotspots[coord].data, numpy_agg[coord].data, equal_nan=True
+        )
+    assert numpy_hotspots.attrs['unit'] == '%'
 
 
 def test_hotspots_dask_numpy(data_hotspots):
     data, kernel, expected_result = data_hotspots
     dask_numpy_agg = create_test_raster(data, backend='dask')
     dask_numpy_hotspots = hotspots(dask_numpy_agg, kernel)
-    general_output_checks(dask_numpy_agg, dask_numpy_hotspots, expected_result)
+    general_output_checks(dask_numpy_agg, dask_numpy_hotspots, expected_result, verify_attrs=False)
+    # validate attrs
+    assert dask_numpy_hotspots.shape == dask_numpy_agg.shape
+    assert dask_numpy_hotspots.dims == dask_numpy_agg.dims
+    for coord in dask_numpy_agg.coords:
+        np.testing.assert_allclose(
+            dask_numpy_hotspots[coord].data, dask_numpy_agg[coord].data, equal_nan=True
+        )
+    assert dask_numpy_hotspots.attrs['unit'] == '%'
 
 
-@pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
+@cuda_and_cupy_available
 def test_hotspot_gpu(data_hotspots):
     data, kernel, expected_result = data_hotspots
     cupy_agg = create_test_raster(data, backend='cupy')
     cupy_hotspots = hotspots(cupy_agg, kernel)
-    general_output_checks(cupy_agg, cupy_hotspots, expected_result)
+    general_output_checks(cupy_agg, cupy_hotspots, expected_result, verify_attrs=False)
+    # validate attrs
+    assert cupy_hotspots.shape == cupy_agg.shape
+    assert cupy_hotspots.dims == cupy_agg.dims
+    for coord in cupy_agg.coords:
+        np.testing.assert_allclose(
+            cupy_hotspots[coord].data, cupy_agg[coord].data, equal_nan=True
+        )
+    assert cupy_hotspots.attrs['unit'] == '%'
 
     # dask + cupy case not implemented
     dask_cupy_agg = create_test_raster(data, backend='dask+cupy')
