@@ -9,10 +9,10 @@ from xrspatial import crop, suggest_zonal_canvas, trim
 from xrspatial import zonal_apply as apply
 from xrspatial import zonal_crosstab as crosstab
 from xrspatial import zonal_stats as stats
-from xrspatial.tests.general_checks import create_test_raster
-from xrspatial.utils import doesnt_have_cuda
 from xrspatial.zonal import regions
 
+from .general_checks import create_test_raster, general_output_checks, has_cuda_and_cupy
+
 
 @pytest.fixture
 def data_zones(backend):
@@ -60,6 +60,40 @@ def result_default_stats():
     return expected_result
 
 
+@pytest.fixture
+def result_default_stats_dataarray():
+    expected_result = np.array(
+        [[[0., 0., 1., 1., 2., 2., 2.4, 2.4],
+          [0., 0., 1., 1., 2., 2., 2.4, 2.4],
+          [0., 0., 1., 1., 2., np.nan, 2.4, 2.4]],
+
+         [[0., 0., 1., 1., 2., 2., 3., 3.],
+          [0., 0., 1., 1., 2., 2., 3., 3.],
+          [0., 0., 1., 1., 2., np.nan, 3., 3.]],
+
+         [[0., 0., 1., 1., 2., 2., 0., 0.],
+          [0., 0., 1., 1., 2., 2., 0., 0.],
+          [0., 0., 1., 1., 2., np.nan, 0., 0.]],
+
+         [[0., 0., 6., 6., 8., 8., 12., 12.],
+          [0., 0., 6., 6., 8., 8., 12., 12.],
+          [0., 0., 6., 6., 8., np.nan, 12., 12.]],
+
+         [[0., 0., 0., 0., 0., 0., 1.2, 1.2],
+          [0., 0., 0., 0., 0., 0., 1.2, 1.2],
+          [0., 0., 0., 0., 0., np.nan, 1.2, 1.2]],
+
+         [[0., 0., 0., 0., 0., 0., 1.44, 1.44],
+          [0., 0., 0., 0., 0., 0., 1.44, 1.44],
+          [0., 0., 0., 0., 0., np.nan, 1.44, 1.44]],
+
+         [[5., 5., 6., 6., 4., 4., 5., 5.],
+          [5., 5., 6., 6., 4., 4., 5., 5.],
+          [5., 5., 6., 6., 4., np.nan, 5., 5.]]]
+    )
+    return expected_result
+
+
 @pytest.fixture
 def result_zone_ids_stats():
     zone_ids = [0, 3]
@@ -76,6 +110,41 @@ def result_zone_ids_stats():
     return zone_ids, expected_result
 
 
+@pytest.fixture
+def result_zone_ids_stats_dataarray():
+    zone_ids = [0, 3]
+    expected_result = np.array(
+        [[[0., 0., np.nan, np.nan, np.nan, np.nan, 2.4, 2.4],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 2.4, 2.4],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 2.4, 2.4]],
+
+         [[0., 0., np.nan, np.nan, np.nan, np.nan, 3., 3.],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 3., 3.],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 3., 3.]],
+
+         [[0., 0., np.nan, np.nan, np.nan, np.nan, 0., 0.],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 0., 0.],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 0., 0.]],
+
+         [[0., 0., np.nan, np.nan, np.nan, np.nan, 12., 12.],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 12., 12.],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 12., 12.]],
+
+         [[0., 0., np.nan, np.nan, np.nan, np.nan, 1.2, 1.2],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 1.2, 1.2],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 1.2, 1.2]],
+
+         [[0., 0., np.nan, np.nan, np.nan, np.nan, 1.44, 1.44],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 1.44, 1.44],
+          [0., 0., np.nan, np.nan, np.nan, np.nan, 1.44, 1.44]],
+
+         [[5., 5., np.nan, np.nan, np.nan, np.nan, 5., 5.],
+          [5., 5., np.nan, np.nan, np.nan, np.nan, 5., 5.],
+          [5., 5., np.nan, np.nan, np.nan, np.nan, 5., 5.]]])
+
+    return zone_ids, expected_result
+
+
 def _double_sum(values):
     return values.sum() * 2
 
@@ -96,6 +165,22 @@ def result_custom_stats():
     return nodata_values, zone_ids, expected_result
 
 
+@pytest.fixture
+def result_custom_stats_dataarray():
+    zone_ids = [1, 2]
+    nodata_values = 0
+    expected_result = np.array(
+        [[[np.nan, np.nan, 12., 12., 16., 16., np.nan, np.nan],
+          [np.nan, np.nan, 12., 12., 16., 16., np.nan, np.nan],
+          [np.nan, np.nan, 12., 12., 16., np.nan, np.nan, np.nan]],
+
+         [[np.nan, np.nan, 0., 0., 0., 0., np.nan, np.nan],
+          [np.nan, np.nan, 0., 0., 0., 0., np.nan, np.nan],
+          [np.nan, np.nan, 0., 0., 0., np.nan, np.nan, np.nan]]]
+    )
+    return nodata_values, zone_ids, expected_result
+
+
 @pytest.fixture
 def result_count_crosstab_2d():
     zone_ids = [1, 2, 3]
@@ -127,11 +212,55 @@ def result_crosstab_3d():
     zone_ids = [1, 2, 3]
     layer = -1
     expected_result = {
-        'zone': [1, 2, 3],
-        'cat1': [6, 5, 6],
-        'cat2': [6, 5, 6],
-        'cat3': [6, 5, 6],
-        'cat4': [6, 5, 6],
+        'mean': {
+            'zone': [1, 2, 3],
+            'cat1': [1., 1., 1.],
+            'cat2': [1., 1., 1.],
+            'cat3': [1., 1., 1.],
+            'cat4': [1., 1., 1.]
+        },
+        'max': {
+            'zone': [1, 2, 3],
+            'cat1': [1., 1., 1.],
+            'cat2': [1., 1., 1.],
+            'cat3': [1., 1., 1.],
+            'cat4': [1., 1., 1.]
+        },
+        'min': {
+            'zone': [1, 2, 3],
+            'cat1': [1., 1., 1.],
+            'cat2': [1., 1., 1.],
+            'cat3': [1., 1., 1.],
+            'cat4': [1., 1., 1.]
+        },
+        'sum': {
+            'zone': [1, 2, 3],
+            'cat1': [6., 5., 6.],
+            'cat2': [6., 5., 6.],
+            'cat3': [6., 5., 6.],
+            'cat4': [6., 5., 6.]
+        },
+        'std': {
+            'zone': [1, 2, 3],
+            'cat1': [0., 0., 0.],
+            'cat2': [0., 0., 0.],
+            'cat3': [0., 0., 0.],
+            'cat4': [0., 0., 0.]
+        },
+        'var': {
+            'zone': [1, 2, 3],
+            'cat1': [0., 0., 0.],
+            'cat2': [0., 0., 0.],
+            'cat3': [0., 0., 0.],
+            'cat4': [0., 0., 0.]
+        },
+        'count': {
+            'zone': [1, 2, 3],
+            'cat1': [6, 5, 6],
+            'cat2': [6, 5, 6],
+            'cat3': [6, 5, 6],
+            'cat4': [6, 5, 6]
+        }
     }
     return layer, zone_ids, expected_result
 
@@ -168,27 +297,56 @@ def check_results(backend, df_result, expected_results_dict):
 
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy', 'cupy'])
 def test_default_stats(backend, data_zones, data_values_2d, result_default_stats):
-    if backend == 'cupy' and doesnt_have_cuda():
-        pytest.skip("CUDA Device not Available")
+    if backend == 'cupy' and not has_cuda_and_cupy():
+        pytest.skip("Requires CUDA and CuPy")
     df_result = stats(zones=data_zones, values=data_values_2d)
     check_results(backend, df_result, result_default_stats)
 
 
+@pytest.mark.parametrize("backend", ['numpy'])
+def test_default_stats_dataarray(
+    backend, data_zones, data_values_2d, result_default_stats_dataarray
+):
+    dataarray_result = stats(
+        zones=data_zones, values=data_values_2d, return_type='xarray.DataArray'
+    )
+    general_output_checks(
+        data_values_2d,
+        dataarray_result,
+        result_default_stats_dataarray,
+        verify_dtype=False,
+        verify_attrs=False,
+    )
+
+
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy', 'cupy'])
 def test_zone_ids_stats(backend, data_zones, data_values_2d, result_zone_ids_stats):
-    if backend == 'cupy' and doesnt_have_cuda():
-        pytest.skip("CUDA Device not Available")
+    if backend == 'cupy' and not has_cuda_and_cupy():
+        pytest.skip("Requires CUDA and CuPy")
     zone_ids, expected_result = result_zone_ids_stats
     df_result = stats(zones=data_zones, values=data_values_2d,
                       zone_ids=zone_ids)
     check_results(backend, df_result, expected_result)
 
 
+@pytest.mark.parametrize("backend", ['numpy'])
+def test_zone_ids_stats_dataarray(
+    backend, data_zones, data_values_2d, result_zone_ids_stats_dataarray
+):
+    zone_ids, expected_result = result_zone_ids_stats_dataarray
+    dataarray_result = stats(
+        zones=data_zones, values=data_values_2d, zone_ids=zone_ids, return_type='xarray.DataArray'
+    )
+    general_output_checks(
+        data_values_2d, dataarray_result, expected_result, verify_dtype=False, verify_attrs=False
+    )
+
+
 @pytest.mark.parametrize("backend", ['numpy', 'cupy'])
 def test_custom_stats(backend, data_zones, data_values_2d, result_custom_stats):
     # ---- custom stats (NumPy and CuPy only) ----
-    if backend == 'cupy' and doesnt_have_cuda():
-        pytest.skip("CUDA Device not Available")
+    if backend == 'cupy' and not has_cuda_and_cupy():
+        pytest.skip("Requires CUDA and CuPy")
 
     custom_stats = {
         'double_sum': _double_sum,
@@ -203,6 +361,23 @@ def test_custom_stats(backend, data_zones, data_values_2d, result_custom_stats):
     check_results(backend, df_result, expected_result)
 
 
+@pytest.mark.parametrize("backend", ['numpy'])
+def test_custom_stats_dataarray(backend, data_zones, data_values_2d, result_custom_stats_dataarray):
+    # ---- custom stats returns a xr.DataArray (NumPy only) ----
+    custom_stats = {
+        'double_sum': _double_sum,
+        'range': _range,
+    }
+    nodata_values, zone_ids, expected_result = result_custom_stats_dataarray
+    dataarray_result = stats(
+        zones=data_zones, values=data_values_2d, stats_funcs=custom_stats,
+        zone_ids=zone_ids, nodata_values=nodata_values, return_type='xarray.DataArray'
+    )
+    general_output_checks(
+        data_values_2d, dataarray_result, expected_result, verify_dtype=False, verify_attrs=False
+    )
+
+
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy'])
 def test_count_crosstab_2d(backend, data_zones, data_values_2d, result_count_crosstab_2d):
     zone_ids, cat_ids, expected_result = result_count_crosstab_2d
@@ -223,11 +398,21 @@ def test_percentage_crosstab_2d(backend, data_zones, data_values_2d, result_perc
 
 
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy'])
-def test_crosstab_3d(backend, data_zones, data_values_3d, result_crosstab_3d):
+def test_crosstab_3d_count(backend, data_zones, data_values_3d, result_crosstab_3d):
     layer, zone_ids, expected_result = result_crosstab_3d
     df_result = crosstab(zones=data_zones, values=data_values_3d,
-                         zone_ids=zone_ids, layer=layer)
-    check_results(backend, df_result, expected_result)
+                         zone_ids=zone_ids, layer=layer, agg='count')
+    check_results(backend, df_result, expected_result['count'])
+
+
+@pytest.mark.parametrize("backend", ['numpy'])
+def test_crosstab_3d_agg_method(backend, data_zones, data_values_3d, result_crosstab_3d):
+    layer, zone_ids, expected_result = result_crosstab_3d
+    agg_methods = ['min', 'max', 'mean', 'sum', 'std', 'var', 'count']
+    for agg in agg_methods:
+        df_result = crosstab(zones=data_zones, values=data_values_3d,
+                             zone_ids=zone_ids, layer=layer, agg=agg)
+        check_results(backend, df_result, expected_result[agg])
 
 
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy'])
