@@ -105,7 +105,7 @@ def _sort_and_stride(zones, values, unique_zones):
     sorted_zones = sorted_zones[np.isfinite(sorted_zones)]
     zone_breaks = _strides(sorted_zones, unique_zones)
 
-    return values_by_zones, zone_breaks
+    return sorted_indices, values_by_zones, zone_breaks
 
 
 def _calc_stats(
@@ -123,8 +123,7 @@ def _calc_stats(
         if unique_zones[i] in zone_ids:
             zone_values = values_by_zones[start:end]
             # filter out non-finite and nodata_values
-            zone_values = zone_values[
-                np.isfinite(zone_values) & (zone_values != nodata_values)]
+            zone_values = zone_values[np.isfinite(zone_values) & (zone_values != nodata_values)]
             if len(zone_values) > 0:
                 results[i] = func(zone_values)
         start = end
@@ -141,13 +140,8 @@ def _single_stats_func(
     nodata_values: Union[int, float] = None,
 ) -> pd.DataFrame:
 
-    values_by_zones, zone_breaks = _sort_and_stride(
-        zones_block, values_block, unique_zones
-    )
-    results = _calc_stats(
-        values_by_zones, zone_breaks,
-        unique_zones, zone_ids, func, nodata_values
-    )
+    _, values_by_zones, zone_breaks = _sort_and_stride(zones_block, values_block, unique_zones)
+    results = _calc_stats(values_by_zones, zone_breaks, unique_zones, zone_ids, func, nodata_values)
     return results
 
 
@@ -224,19 +218,15 @@ def _stats_dask_numpy(
         stats_dict['mean'] = _dask_mean(stats_dict['sum'], stats_dict['count'])
     if 'std' in stats_funcs:
         stats_dict['std'] = _dask_std(
-            stats_dict['sum_squares'], stats_dict['sum'] ** 2,
-            stats_dict['count']
+            stats_dict['sum_squares'], stats_dict['sum'] ** 2, stats_dict['count']
         )
     if 'var' in stats_funcs:
         stats_dict['var'] = _dask_var(
-            stats_dict['sum_squares'], stats_dict['sum'] ** 2,
-            stats_dict['count']
+            stats_dict['sum_squares'], stats_dict['sum'] ** 2, stats_dict['count']
         )
 
     # generate dask dataframe
-    stats_df = dd.concat(
-        [dd.from_dask_array(s) for s in stats_dict.values()], axis=1
-    )
+    stats_df = dd.concat([dd.from_dask_array(s) for s in stats_dict.values()], axis=1)
     # name columns
     stats_df.columns = stats_dict.keys()
     # select columns
@@ -259,7 +249,8 @@ def _stats_numpy(
     zone_ids: List[Union[int, float]],
     stats_funcs: Dict,
     nodata_values: Union[int, float],
-) -> pd.DataFrame:
+    return_type: str,
+) -> Union[pd.DataFrame, np.ndarray]:
 
     # find ids for all zones
     unique_zones = np.unique(zones[np.isfinite(zones)])
@@ -271,23 +262,40 @@ def _stats_numpy(
         # remove zones that do not exist in `zones` raster
         zone_ids = [z for z in zone_ids if z in unique_zones]
 
-    selected_indexes = [i for i, z in enumerate(unique_zones) if z in zone_ids]
-    values_by_zones, zone_breaks = _sort_and_stride(
-        zones, values, unique_zones
-    )
+    sorted_indices, values_by_zones, zone_breaks = _sort_and_stride(zones, values, unique_zones)
+    if return_type == 'pandas.DataFrame':
+        stats_dict = {}
+        stats_dict["zone"] = zone_ids
+        selected_indexes = [i for i, z in enumerate(unique_zones) if z in zone_ids]
+        for stats in stats_funcs:
+            func = stats_funcs.get(stats)
+            stats_dict[stats] = _calc_stats(
+                values_by_zones, zone_breaks,
+                unique_zones, zone_ids, func, nodata_values
+            )
+            stats_dict[stats] = stats_dict[stats][selected_indexes]
+        result = pd.DataFrame(stats_dict)
 
-    stats_dict = {}
-    stats_dict["zone"] = zone_ids
-    for stats in stats_funcs:
-        func = stats_funcs.get(stats)
-        stats_dict[stats] = _calc_stats(
-            values_by_zones, zone_breaks,
-            unique_zones, zone_ids, func, nodata_values
-        )
-        stats_dict[stats] = stats_dict[stats][selected_indexes]
-
-    stats_df = pd.DataFrame(stats_dict)
-    return stats_df
+    else:
+        result = np.full((len(stats_funcs), values.size), np.nan)
+        zone_ids_map = {z: i for i, z in enumerate(unique_zones) if z in zone_ids}
+        stats_id = 0
+        for stats in stats_funcs:
+            func = stats_funcs.get(stats)
+            stats_results = _calc_stats(
+                values_by_zones, zone_breaks,
+                unique_zones, zone_ids, func, nodata_values
+            )
+            for zone in zone_ids:
+                iz = zone_ids_map[zone]  # position of zone in unique_zones
+                if iz == 0:
+                    zs = sorted_indices[: zone_breaks[iz]]
+                else:
+                    zs = sorted_indices[zone_breaks[iz-1]: zone_breaks[iz]]
+                result[stats_id][zs] = stats_results[iz]
+            stats_id += 1
+        result = result.reshape(len(stats_funcs), *values.shape)
+    return result
 
 
 def _stats_cupy(
@@ -391,7 +399,8 @@ def stats(
         "count",
     ],
     nodata_values: Union[int, float] = None,
-) -> Union[pd.DataFrame, dd.DataFrame]:
+    return_type: str = 'pandas.DataFrame',
+) -> Union[pd.DataFrame, dd.DataFrame, xr.DataArray]:
     """
     Calculate summary statistics for each zone defined by a `zones`
     dataset, based on `values` aggregate.
@@ -438,6 +447,11 @@ def stats(
         Cells with `nodata_values` do not belong to any zone,
         and thus excluded from calculation.
 
+    return_type: str, default='pandas.DataFrame'
+        Format of returned data. If `zones` and `values` numpy backed xarray DataArray,
+        allowed values are 'pandas.DataFrame', and 'xarray.DataArray'.
+        Otherwise, only 'pandas.DataFrame' is supported.
+
     Returns
     -------
     stats_df : Union[pandas.DataFrame, dask.dataframe.DataFrame]
@@ -503,9 +517,8 @@ def stats(
     stats() works with Dask with NumPy backed DataArray
         >>> import dask.array as da
         >>> import dask.array as da
-        >>> values_dask = xr.DataArray(da.from_array(values, chunks=(3, 3)))
-        >>> zones_dask = xr.DataArray(da.from_array(zones, chunks=(3, 3)))
-
+        >>> values_dask = xr.DataArray(da.from_array(values_data, chunks=(3, 3)))
+        >>> zones_dask = xr.DataArray(da.from_array(zones_data, chunks=(3, 3)))
         >>> # Calculate Stats with dask backed xarray DataArrays
         >>> dask_stats_df = stats(zones=zones_dask, values=values_dask)
         >>> print(type(dask_stats_df))
@@ -516,20 +529,6 @@ def stats(
         1    10  27.0   49    5   675  14.21267  202.0     25
         2    20  72.0   94   50  1800  14.21267  202.0     25
         3    30  77.0   99   55  1925  14.21267  202.0     25
-
-        >>> # Custom Stats with dask backed xarray DataArrays
-        >>> dask_custom_stats ={'double_sum': lambda val: val.sum()*2}
-        >>> dask_custom_stats_df = stats(
-        >>>      zones=zones_dask, values=values_dask, stats_funcs=custom_stats
-        >>> )
-        >>> print(type(dask_custom_stats_df))
-        <class 'dask.dataframe.core.DataFrame'>
-        >>> print(dask_custom_stats_df.compute())
-            zone  double_sum
-        0     0        1100
-        1    10        1350
-        2    20        3600
-        3    30        3850
     """
 
     validate_arrays(zones, values)
@@ -568,17 +567,25 @@ def stats(
         stats_funcs_dict = stats_funcs.copy()
 
     mapper = ArrayTypeFunctionMapping(
-        numpy_func=_stats_numpy,
+        numpy_func=lambda *args: _stats_numpy(*args, return_type=return_type),
         dask_func=_stats_dask_numpy,
         cupy_func=_stats_cupy,
         dask_cupy_func=lambda *args: not_implemented_func(
             *args, messages='stats() does not support dask with cupy backed DataArray'  # noqa
         ),
     )
-    stats_df = mapper(values)(
-        zones.data, values.data, zone_ids, stats_funcs_dict, nodata_values
+    result = mapper(values)(
+        zones.data, values.data, zone_ids, stats_funcs_dict, nodata_values,
     )
-    return stats_df
+
+    if return_type == 'xarray.DataArray':
+        return xr.DataArray(
+            result,
+            coords={'stats': list(stats_funcs_dict.keys()), **values.coords},
+            dims=('stats', *values.dims),
+            attrs=values.attrs
+        )
+    return result
 
 
 def _find_cats(values, cat_ids, nodata_values):
@@ -612,46 +619,51 @@ def _get_zone_values(values_by_zones, start, end):
         return values_by_zones[:, start:end]
 
 
-def _single_zone_crosstab(
+def _single_zone_crosstab_2d(
     zone_values,
     unique_cats,
     cat_ids,
     nodata_values,
     crosstab_dict,
 ):
-    if len(zone_values.shape) == 1:
-        # 1D flatten, i.e, original data is 2D
+    # 1D flatten zone_values, i.e, original data is 2D
+    # filter out non-finite and nodata_values
+    zone_values = zone_values[
+        np.isfinite(zone_values) & (zone_values != nodata_values)
+    ]
+    total_count = zone_values.shape[0]
+    crosstab_dict[TOTAL_COUNT].append(total_count)
 
-        # filter out non-finite and nodata_values
-        zone_values = zone_values[
-            np.isfinite(zone_values) & (zone_values != nodata_values)
-        ]
-        total_count = zone_values.shape[0]
-        crosstab_dict[TOTAL_COUNT].append(total_count)
+    sorted_zone_values = np.sort(zone_values)
+    zone_cat_breaks = _strides(sorted_zone_values, unique_cats)
 
-        sorted_zone_values = np.sort(zone_values)
-        zone_cat_breaks = _strides(sorted_zone_values, unique_cats)
+    cat_start = 0
 
-        cat_start = 0
+    for j, cat in enumerate(unique_cats):
+        if cat in cat_ids:
+            count = zone_cat_breaks[j] - cat_start
+            crosstab_dict[cat].append(count)
+            cat_start = zone_cat_breaks[j]
 
-        for j, cat in enumerate(unique_cats):
-            if cat in cat_ids:
-                count = zone_cat_breaks[j] - cat_start
-                crosstab_dict[cat].append(count)
-                cat_start = zone_cat_breaks[j]
 
-    else:
-        # 2D flatten, i.e, original data is 3D
-        for j, cat in enumerate(unique_cats):
-            if cat in cat_ids:
-                zone_cat_data = zone_values[j]
-                # filter out non-finite and nodata_values
-                zone_cat_data = zone_cat_data[
-                    np.isfinite(zone_cat_data)
-                    & (zone_cat_data != nodata_values)
-                ]
-                count = zone_cat_data.shape[0]
-                crosstab_dict[cat].append(count)
+def _single_zone_crosstab_3d(
+    zone_values,
+    unique_cats,
+    cat_ids,
+    nodata_values,
+    crosstab_dict,
+    stats_func
+):
+    # 2D flatten `zone_values`, i.e, original data is 3D
+    for j, cat in enumerate(unique_cats):
+        if cat in cat_ids:
+            zone_cat_data = zone_values[j]
+            # filter out non-finite and nodata_values
+            zone_cat_data = zone_cat_data[
+                np.isfinite(zone_cat_data)
+                & (zone_cat_data != nodata_values)
+            ]
+            crosstab_dict[cat].append(stats_func(zone_cat_data))
 
 
 def _crosstab_numpy(
@@ -680,7 +692,7 @@ def _crosstab_numpy(
     for cat in cat_ids:
         crosstab_dict[cat] = []
 
-    values_by_zones, zone_breaks = _sort_and_stride(
+    _, values_by_zones, zone_breaks = _sort_and_stride(
         zones, values, unique_zones
     )
 
@@ -690,10 +702,14 @@ def _crosstab_numpy(
         if unique_zones[i] in zone_ids:
             # get data for zone unique_zones[i]
             zone_values = _get_zone_values(values_by_zones, start, end)
-            _single_zone_crosstab(
-                zone_values, unique_cats, cat_ids,
-                nodata_values, crosstab_dict
-            )
+            if len(values.shape) == 2:
+                _single_zone_crosstab_2d(
+                    zone_values, unique_cats, cat_ids, nodata_values, crosstab_dict
+                )
+            else:
+                _single_zone_crosstab_3d(
+                    zone_values, unique_cats, cat_ids, nodata_values, crosstab_dict, _DEFAULT_STATS[agg]  # noqa
+                )
         start = end
 
     if TOTAL_COUNT in crosstab_dict:
@@ -731,7 +747,7 @@ def _single_chunk_crosstab(
     for cat in cat_ids:
         results[cat] = []
 
-    values_by_zones, zone_breaks = _sort_and_stride(
+    _, values_by_zones, zone_breaks = _sort_and_stride(
         zones_block, values_block, unique_zones
     )
 
@@ -741,10 +757,15 @@ def _single_chunk_crosstab(
         if unique_zones[i] in zone_ids:
             # get data for zone unique_zones[i]
             zone_values = _get_zone_values(values_by_zones, start, end)
-            _single_zone_crosstab(
-                zone_values, unique_cats, cat_ids,
-                nodata_values, results
-            )
+            if len(values_block.shape) == 2:
+                _single_zone_crosstab_2d(
+                    zone_values, unique_cats, cat_ids, nodata_values, results
+                )
+            else:
+                _single_zone_crosstab_3d(
+                    zone_values, unique_cats, cat_ids, nodata_values, results, _DEFAULT_STATS['count']  # noqa
+                )
+
         start = end
 
     if TOTAL_COUNT in results:
@@ -874,8 +895,10 @@ def crosstab(
 
     agg: str, default = 'count'
         Aggregation method.
-        If the data is 2D, available options are: percentage, count.
-        If the data is 3D, available option is: count.
+        If the `values` data is 2D, available options are: `percentage`, and `count`.
+        If `values` is 3D and is numpy-backed, available options are:
+        `min`, `max`, `mean`, `sum`, `std`, `var`, and `count`.
+        If `values` is 3D and is dask with numpy-backed, the only available option is `count`.
 
     nodata_values: int, float, default=None
         Nodata value in `values` raster.
@@ -930,10 +953,10 @@ def crosstab(
     .. sourcecode:: python
 
         >>> import dask.array as da
-        >>> values_dask = xr.DataArray(da.from_array(values, chunks=(3, 3)))
-        >>> zones_dask = xr.DataArray(da.from_array(zones, chunks=(3, 3)))
-        >>> df = crosstab(zones=zones_dask, values=values_dask)
-        >>> print(df)
+        >>> values_dask = xr.DataArray(da.from_array(values_data, chunks=(3, 3)))
+        >>> zones_dask = xr.DataArray(da.from_array(zones_data, chunks=(3, 3)))
+        >>> dask_df = crosstab(zones=zones_dask, values=values_dask)
+        >>> print(dask_df)
             Dask DataFrame Structure:
             zone    0.0 10.0    20.0    30.0    40.0    50.0
             npartitions=5
@@ -943,7 +966,7 @@ def crosstab(
             4   ... ... ... ... ... ... ...
             5   ... ... ... ... ... ... ...
             Dask Name: astype, 1186 tasks
-        >>> print(dask_df.compute)
+        >>> print(dask_df.compute())
                 zone  0.0  10.0  20.0  30.0  40.0  50.0
             0      0    1     0     0     0     0     0
             1      1    3     0     0     0     0     0
@@ -977,17 +1000,23 @@ def crosstab(
         raise ValueError("`values` must use either 2D or 3D coordinates.")
 
     agg_2d = ["percentage", "count"]
-    agg_3d = ["count"]
+    agg_3d_numpy = _DEFAULT_STATS.keys()
+    agg_3d_dask = ["count"]
 
     if values.ndim == 2 and agg not in agg_2d:
         raise ValueError(
             f"`agg` method for 2D data array must be one of following {agg_2d}"
         )
 
-    if values.ndim == 3 and agg not in agg_3d:
-        raise ValueError(
-            f"`agg` method for 3D data array must be one of following {agg_3d}"
-        )
+    if values.ndim == 3:
+        if isinstance(values.data, np.ndarray) and agg not in agg_3d_numpy:
+            raise ValueError(
+                f"`agg` method for 3D numpy backed data array must be one of following {agg_3d_numpy}"  # noqa
+            )
+        if isinstance(values.data, da.Array) and agg not in agg_3d_dask:
+            raise ValueError(
+                f"`agg` method for 3D dask backed data array must be one of following {agg_3d_dask}"
+            )
 
     if len(values.shape) == 3:
         # 3D case
@@ -1076,6 +1105,7 @@ def apply(
 
         >>> import numpy as np
         >>> import xarray as xr
+        >>> from xrspatial.zonal import apply
         >>> zones_val = np.array([
             [1, 1, 0, 2],
             [0, 2, 1, 2]])
@@ -1088,7 +1118,7 @@ def apply(
         >>> apply(zones, agg, func)
         >>> agg
         array([[0, 0, 5, 0],
-               [3, 0, 0, 0]])
+               [3, np.nan, 0, 0]])
     """
     if not isinstance(zones, xr.DataArray):
         raise TypeError("zones must be instance of DataArray")
@@ -1232,6 +1262,7 @@ def suggest_zonal_canvas(
         >>> from spatialpandas import GeoDataFrame
         >>> import geopandas as gpd
         >>> import datashader as ds
+        >>> from xrspatial.zonal import suggest_zonal_canvas
 
         >>> df = gpd.read_file(gpd.datasets.get_path('naturalearth_lowres'))
         >>> df = df.to_crs("EPSG:3857")
@@ -1254,12 +1285,16 @@ def suggest_zonal_canvas(
                 crs='Mercator',
                 min_pixels=min_pixels,
             )
+        >>> height, width
+        (1537, 2376)
         >>> cvs = ds.Canvas(x_range=x_range, y_range=y_range,
         >>>             plot_height=height, plot_width=width)
         >>> spatial_df = GeoDataFrame(df, geometry='geometry')
         >>> agg = cvs.polygons(spatial_df, 'geometry', agg=ds.max('id'))
         >>> min_poly_id = df.area.argmin()
         >>> actual_min_pixels = len(np.where(agg.data==min_poly_id)[0])
+        >>> actual_min_pixels
+        22
     """
     full_xrange, full_yrange = get_full_extent(crs)
     xmin, xmax = full_xrange
